/*
   CancelDynLots
      Mit zwei Inputs
   Es wird ber Positionen mit MagicNummer MagicNumberCompensatory und MagicNumberCompensatoryM1
      gelaufen,
      fr jede Position errechne Profit in AccCurr
      wenn profit <= CancelDynLotsLoss 
         Schliee Position
*/
void CancelDynLots()
{
   int comp[], compM1[];
   int compCnt = FillArrWithTickets(comp, MagicNumberCompensatory, Symbol(), MODE_ALL, MODE_TRADES);
   int compM1Cnt = FillArrWithTickets(compM1, MagicNumberCompensatoryM1, Symbol(), MODE_ALL, MODE_TRADES);
   
   compCnt = IntAddIntArr(comp, compM1);
   
   for(int i=0; i < compCnt; i++)
   {
      int ticket = comp[i];
      
      double profit = 0;
      bool reached = false;
      
      double profitCurr = GetTradesProfitInAccCurr(ticket);
      double profitPoints = GetTradesProfitInPoints(ticket);
      
      if(CancelDynLotsLossCurrency < 0)
         reached = (profitCurr < CancelDynLotsLossCurrency);
      else
         reached = (profitPoints < CancelDynLotsLossPoints);
      
      if(reached)
      {
         Print("cancelling comp Trade, profit="+profitCurr+" profitPoints="+profitPoints+
            " CancelDynLotsLossCurrency="+CancelDynLotsLossCurrency+
            " CancelDynLotsLossPoints="+CancelDynLotsLossPoints);
            
         CloseOrder(ticket);
         LossFromCompCancel += profitCurr;
         
         Print("updating LossFromCompCancel to "+DoubleToStr(LossFromCompCancel, 2));
      }
   }
}

/*
   // Soweit mehrere offene Open (von einer (!) Sorte, d.h. Long bzw. Short) bestehen, werden fr diese intern folgende Variablen mitgefhrt:
   // - Durchschnittlicher_Kaufpreis = Kaufpreis / Anzahl
   // - Durchschnnittlicher_Take_Profit_Kurs = Durchschnnittlicher_Kaufpreis + (TakeProfit_Moduliert / Anzahl)
   // - TakeProfit_Moduliert = TakeProfit + ProfitPointsAvgTp (Default: ProfitPointsAvgTp = 0))
   // Soweit der akutelle Kurs den Durchschnittlicher_Take_Profit_Kurs erreichen solle, werden ALLE betreffenden Open-Positionen zeitgleich aufgelst.
   
*/
void ManageAvgTpClosing()
{
   if(UseUnweightedAvgTp())
   {
      CloseByUnweightedAvgTpLong();
      CloseByUnweightedAvgTpShort();
   }
   
   if(UseDynLotsAvgTp())
   {
      CloseByDynLotsAvgTp(OP_BUY);
      CloseByDynLotsAvgTp(OP_SELL);
   }
   
   if(UseM1Hedging)
   {
      CloseByDynLotsAvgTp_x(OP_BUY);
      CloseByDynLotsAvgTp_x(OP_SELL);
   }
}

void CloseByDynLotsAvgTp(int cmd)
{
   int tickets[];
   int ticketCount = FillArrWithDynLotsPool(tickets, cmd, CurPhaseStart);
   
   double profit = GetCumulatedProfit(tickets, MODE_ACCOUNT_CURRENCY)+LossFromCompCancel;
   
   if(ticketCount > 0)
   if(profit >= 0)
   {
      Print("closing "+ticketCount+" trade due to dynamic average TP closing...");
      Print("setting LossFromCompCancel from "+DoubleToStr(LossFromCompCancel,2)+" to 0...");
      LossFromCompCancel = 0;
         
      CloseOrders(tickets);
   }
}

void CloseByDynLotsAvgTp_x(int cmd)
{
   int temp[];
   int tempCnt = FillArrWithTickets(temp, MagicNumberCompensatoryM1, Symbol(), cmd, MODE_TRADES);
   
   if(tempCnt <= 0)
      return;

   int tickets[];
   int ticketCount = FillArrWithDynLotsPool_x(tickets, cmd, CurPhaseStart_x);
   
   double profit = GetCumulatedProfit(tickets, MODE_ACCOUNT_CURRENCY)+LossFromCompCancel;
   
   if(ticketCount > 0)
   if(profit >= 0)
   {
      Print("closing "+ticketCount+" trade due to dynamic average TP closing...");
      Print("setting LossFromCompCancel from "+DoubleToStr(LossFromCompCancel,2)+" to 0...");
      LossFromCompCancel = 0;
         
      CloseOrders(tickets);
   }
}

void CloseByUnweightedAvgTpLong()
{
   int longs[];
   int mode = MODE_LONG;
   //int longCount = FillArrWithTickets(longs, MagicNumber, Symbol(), mode, MODE_TRADES);
   int longCount = FillArrWithStaticAvgTpPool(longs, mode);
   
   if(longCount < 2)
   {
      HideAvgTp(OP_BUY);
      //return;
   }
      
 //safety measure
   if(longCount > 0)
   {
      double avgOpenPrice = GetAverageOpenPriceForArr(longs);
      
      double tp = avgOpenPrice + ProfitPointsAvgTp*Point;
      
      ShowAvgTp(OP_BUY, tp);
      
      if(!UseIndividualStop)
         ModifyTakeProfits(longs, 0);
      
      if(Bid >= tp)
      {
         Print("closing "+ArraySize(longs)+" trade due to static average TP closing...");
         
         CloseOrders(longs);
      }
   }
   else
      HideAvgTp(OP_BUY);
}

void CloseByUnweightedAvgTpShort()
{
   int shorts[];
   int mode = MODE_SHORT;
   //int shortCount = FillArrWithTickets(shorts, MagicNumber, Symbol(), mode, MODE_TRADES);
   int shortCount = FillArrWithStaticAvgTpPool(shorts, mode);

   if(shortCount < 2)
   {
      HideAvgTp(OP_SELL);
      //return;
   }
      
 //safety measure
   if(shortCount > 0)
   {
      double avgOpenPrice = GetAverageOpenPriceForArr(shorts);
   
      double tp = avgOpenPrice - ProfitPointsAvgTp*Point;
      
      ShowAvgTp(OP_SELL, tp);
      
      if(!UseIndividualStop)
         ModifyTakeProfits(shorts, 0);
      
      if(Ask <= tp)
      {
         Print("closing "+ArraySize(shorts)+" trade due to static average TP closing...");
         
         CloseOrders(shorts);
      }
   }
   else
      HideAvgTp(OP_SELL);
}

void OpenDynLotsTrade(int curPhase)
{
   if(!UseDynLotsAvgTp() || curPhase < 0)
      return;

   int tradesCountContainer;
   double curProfitContainer;
   double dynLots = GetDynLots(curPhase, CurPhaseStart, tradesCountContainer, curProfitContainer);
   
   Print("calculated dynLots="+dynLots);
   
   int ticket = -1;
   if(dynLots > 0)
   {
      Print("opening compensatory trade for "+tradesCountContainer+" trades, cumulated loss: "+curProfitContainer+"...");
      ticket = OpenCompensatoryTrade(curPhase, dynLots, MagicNumberCompensatory);
   }
}

void OpenDynLotsTrade_x(int curPhase_x)
{
   if(!UseM1Hedging || curPhase_x < 0)
      return;

   int ticketsH[];
   int cntH = FillArrWithTickets(ticketsH, MagicNumberHedge, Symbol(), ReverseCmd(curPhase_x));
   
   if(cntH <= 0)
      return;
   
   int tradesCountContainer;
   double curProfitContainer;
   double dynLots = GetDynLots_x(curPhase_x, CurPhaseStart_x, tradesCountContainer, curProfitContainer);
   
   Print("calculated M1 dynLots="+dynLots);
   
   int ticket = -1;
   if(dynLots > 0)
   {
      Print("opening compensatory trade for "+tradesCountContainer+" trades, cumulated loss: "+curProfitContainer+"...");
      ticket = OpenCompensatoryTrade(curPhase_x, dynLots, MagicNumberCompensatoryM1);
   }
}

double GetDynLots(int curPhase, datetime curPhaseStart, int& tradesCount, double& curProfit)
{
   int tickets[];
   int ticketCount = FillArrWithDynLotsPool(tickets, curPhase, curPhaseStart);
   tradesCount = ticketCount;
   
   /*
      Berechnung der dyn lots:
      - errechnen des aktuell floatenden Profit in AccCurrency
         - wenn >= 0, dann nix machen
      - ermitteln: 
         - pipValueOneLot = wert eines Pips bei einem Lot
         - dynLots = ( abs(profit) / (pipValueOneLot * ppDynLot) )
   */
   double profit = GetCumulatedProfit(tickets, MODE_ACCOUNT_CURRENCY)+LossFromCompCancel;
   curProfit = profit;
   
   if(profit >= 0)
      return(0);
      
   double pipValueOneLot = MarketInfo(Symbol(), MODE_TICKVALUE);
   //Print("pipValueOneLot="+pipValueOneLot+" profit="+profit);
   double dynLots = ( MathAbs(profit) / (pipValueOneLot * ProfitPointsDynLots) );
   
   dynLots = RoundLots(dynLots, ROUND_CEIL);
   
   return(dynLots);
}

double GetDynLots_x(int curPhase, datetime curPhaseStart, int& tradesCount, double& curProfit)
{
   int tickets[];
   int ticketCount = FillArrWithDynLotsPool_x(tickets, curPhase, curPhaseStart);
   tradesCount = ticketCount;
   
   /*
      Berechnung der dyn lots:
      - errechnen des aktuell floatenden Profit in AccCurrency
         - wenn >= 0, dann nix machen
      - ermitteln: 
         - pipValueOneLot = wert eines Pips bei einem Lot
         - dynLots = ( abs(profit) / (pipValueOneLot * ppDynLot) )
   */
   double profit = GetCumulatedProfit(tickets, MODE_ACCOUNT_CURRENCY)+LossFromCompCancel;
   curProfit = profit;
   
   if(profit >= 0)
      return(0);
      
   double pipValueOneLot = MarketInfo(Symbol(), MODE_TICKVALUE);
   //Print("pipValueOneLot="+pipValueOneLot+" profit="+profit);
   double dynLots = ( MathAbs(profit) / (pipValueOneLot * ProfitPointsDynLots) );
   
   dynLots = RoundLots(dynLots, ROUND_CEIL);
   
   return(dynLots);
}

int FillArrWithDynLotsPool_x(int tickets[], int curPhase, datetime curPhaseStart)
{
   int tempTickets[], compTrades[], ticketsH[], tempTicketsH[];
   ArrayResize(ticketsH, 0);
   ArrayResize(tempTickets, 0);
   ArrayResize(tempTicketsH, 0);
   ArrayResize(tickets, 0);
   ArrayResize(compTrades, 0);
   
   int ticketCount = FillArrWithTickets(tempTickets, MagicNumber, Symbol(), curPhase, MODE_TRADES);
   int ticketCountH = FillArrWithTickets(tempTicketsH, MagicNumberHedge, Symbol(), ReverseCmd(curPhase), MODE_TRADES);
   int compCount = FillArrWithTickets(compTrades, MagicNumberCompensatoryM1, Symbol(), curPhase, MODE_TRADES);
   
   ticketCount = GetTradesOpenedBetween(0, curPhaseStart-1, tempTickets, tickets);
   ticketCount = GetTradesOpenedBetween(0, curPhaseStart-1, tempTicketsH, ticketsH);
   ticketCount = IntAddIntArr(tickets, compTrades);
   ticketCount = IntAddIntArr(tickets, ticketsH);
   
   return(ticketCount);
}

/*
   Pool aus:
      "normalen" Trades der aktuellen PhaseRtg.
      Hedge-Trades der gegenlufigen Phase, also Hedge Trades fr die "normalen" Trades aus der aktuellen PhaseRtg.
      komp. Trades der aktuellen PhaseRtg.
*/
int FillArrWithDynLotsPool(int tickets[], int curPhase, datetime curPhaseStart)
{
   int tempTickets[], compTrades[], ticketsH[], tempTicketsH[];
   ArrayResize(ticketsH, 0);
   ArrayResize(tempTickets, 0);
   ArrayResize(tempTicketsH, 0);
   ArrayResize(tickets, 0);
   ArrayResize(compTrades, 0);
   
   int ticketCount = FillArrWithTickets(tempTickets, MagicNumber, Symbol(), curPhase, MODE_TRADES);
   int ticketCountH = FillArrWithTickets(tempTicketsH, MagicNumberHedge, Symbol(), ReverseCmd(curPhase), MODE_TRADES);
   int compCount = FillArrWithTickets(compTrades, MagicNumberCompensatory, Symbol(), curPhase, MODE_TRADES);
   
 //tickets wird mit "normalen" Trades der aktuellen PhasenRtg. gefllt
   ticketCount = GetTradesOpenedBetween(0, curPhaseStart-1, tempTickets, tickets);
   ticketCount = GetTradesOpenedBetween(0, curPhaseStart-1, tempTicketsH, ticketsH);
 //tickets wird mit kompTrades verschnitten
   ticketCount = IntAddIntArr(tickets, compTrades);
 //ticktes wird mit HedgeTickets verschnitten
   ticketCount = IntAddIntArr(tickets, ticketsH);
   
   return(ticketCount);
}

/*
#define BUILD_POOL_METHOD_AFTER_PHASE "AFTER_PHASE"
#define BUILD_POOL_METHOD_AFTER_CANDLE "AFTER_CANDLE"
#define BUILD_POOL_METHOD_ALL "ALL"

extern string D = "-- Trade Pool Bildung (AFTER_PHASE, AFTER_CANDLE, ALL) --";
extern string BuildTradePoolMethod = "AFTER_CANDLE";
extern bool DeleteIndividualStop = false; // gibt an, ob der individuelle Stopp gelscht werden soll, wenn der AvgTP berechnet wird.
*/
int FillArrWithStaticAvgTpPool(int tickets[], int curPhase)
{
   ArrayResize(tickets, 0);
   
   int ticketCount;
   int tempTickets[];
   if(BuildTradePoolMethod == BUILD_POOL_METHOD_ALL)
   {
      ticketCount = FillArrWithTickets(tickets, MagicNumber, Symbol(), curPhase, MODE_TRADES);
   }
   else if(BuildTradePoolMethod == BUILD_POOL_METHOD_AFTER_PHASE)
   {
      ticketCount = FillArrWithTickets(tempTickets, MagicNumber, Symbol(), curPhase, MODE_TRADES);
      ticketCount = GetTradesOpenedBetween(0, CurPhaseStart-1, tempTickets, tickets);
   }
   else if(BuildTradePoolMethod == BUILD_POOL_METHOD_AFTER_CANDLE)
   {
      ticketCount = FillArrWithTickets(tempTickets, MagicNumber, Symbol(), curPhase, MODE_TRADES);
      ticketCount = GetTradesOpenedBetween(0, Time[0]-1, tempTickets, tickets);
   }
   
   return(ArraySize(tickets));
}

bool UseUnweightedAvgTp()
{
   //return(AverageTPMethod == AVG_TP_METHOD_STATIC);
   return(UseStaticAverageTP);
}

bool UseDynLotsAvgTp()
{
   //return(AverageTPMethod == AVG_TP_METHOD_DYNAMIC);
   return(UseDynamicLotsAverageTP);
}

double GetAverageOpenPriceForArr(int tickets[])
{
   double avg = 0, sum = 0, validOrders = 0;
   
   for(int i=0; i < ArraySize(tickets); i++)
   {
      if(!OrderSelect(tickets[i], SELECT_BY_TICKET))
         continue;
         
      sum += OrderOpenPrice();
      validOrders++;
   }
   
   if(validOrders > 0)
      avg = sum/validOrders;
   
   return(avg);
}

/*
Send AvgTP to Broker which results in INVALID STOPS in some cases

void CloseByUnweightedAvgTpLong()
{
   int longs[];
   int mode = MODE_LONG;
   int longCount = FillArrWithTickets(longs, MagicNumber, Symbol(), mode, MODE_TRADES);
   
   if(longCount < 2)
      return;
      
   int lastOrder = GetLastOpenedTicket(MagicNumber, Symbol(), mode, MODE_TRADES);
   longCount = IntDelValue(longs, lastOrder);
   
 //safety measure
   if(longCount > 0)
   {
      double avgOpenPrice = GetAverageOpenPriceForArr(longs);
      
      double tp = avgOpenPrice + ProfitPointsAvgTp*Point;
      
      ModifyTakeProfits(longs, tp);
   }
}

void CloseByUnweightedAvgTpShort()
{
   int shorts[];
   int mode = MODE_SHORT;
   int shortCount = FillArrWithTickets(shorts, MagicNumber, Symbol(), mode, MODE_TRADES);

   if(shortCount < 2)
      return;
      
   int lastOrder = GetLastOpenedTicket(MagicNumber, Symbol(), mode, MODE_TRADES);
   shortCount = IntDelValue(shorts, lastOrder);
   
 //safety measure
   if(shortCount > 0)
   {
      double avgOpenPrice = GetAverageOpenPriceForArr(shorts);
   
      double tp = avgOpenPrice - ProfitPointsAvgTp*Point;
      
      ModifyTakeProfits(shorts, tp);
   }
}*/

double GetBiggestLoserCompLots(int cmd, int& ticketContainer)
{
   int tickets[], ticketsM1[];
   int ticketCount = FillArrWithDynLotsPool(tickets, cmd, TimeCurrent()+1);
   int ticketCountM1 = FillArrWithDynLotsPool_x(ticketsM1, cmd, TimeCurrent()+1);
   
   ticketCount = IntAddIntArr(tickets, ticketsM1);
   
   int ticketBiggestLoser = -1;
   
   if(ticketCount > 0)
      ticketBiggestLoser = GetBiggestLoser(tickets);
   else
      return(0);
      
   ticketContainer = ticketBiggestLoser;
   
   /*
      Berechnung der dyn lots:
      - errechnen des aktuell floatenden Profit in AccCurrency
         - wenn >= 0, dann nix machen
      - ermitteln: 
         - pipValueOneLot = wert eines Pips bei einem Lot
         - dynLots = ( abs(profit) / (pipValueOneLot * ppDynLot) )
   */
   double profit = GetTradesProfitInAccCurr(ticketBiggestLoser);
   
   if(profit >= 0)
      return(0);
      
   double pipValueOneLot = MarketInfo(Symbol(), MODE_TICKVALUE);
   //Print("pipValueOneLot="+pipValueOneLot+" profit="+profit);
   double dynLots = ( MathAbs(profit) / (pipValueOneLot * ProfitPointsDynLots) );
   
   dynLots = RoundLots(dynLots, ROUND_CEIL);
   
   return(dynLots);
}

void OpenStaticKompTrade(int cmd)
{
 //erstellt arrays aus den ganzen Trades (normal + comps, open and historical)
   int t1[], t2[], t3[], t4[], t5[], t6[], t7[];
   FillArrWithTickets(t1, MagicNumber, Symbol(), cmd, MODE_TRADES);
   FillArrWithTickets(t2, MagicNumberCompensatory, Symbol(), cmd, MODE_TRADES);
   FillArrWithTickets(t3, MagicNumberCompensatoryM1, Symbol(), cmd, MODE_TRADES);
   FillArrWithTickets(t4, MagicNumber, Symbol(), cmd, MODE_HISTORY);
   FillArrWithTickets(t5, MagicNumberCompensatory, Symbol(), cmd, MODE_HISTORY);
   FillArrWithTickets(t6, MagicNumberCompensatoryM1, Symbol(), cmd, MODE_HISTORY);
   
 //merge arrays so that you have all trades in t1
   IntAddIntArr(t1, t2);
   IntAddIntArr(t1, t3);
   IntAddIntArr(t1, t4);
   IntAddIntArr(t1, t5);
   int count = IntAddIntArr(t1, t6);
   
 //filter out trades closed before current EA's start time
   count = GetTradesClosedSince(t1, t7, EaStartTime);
   
 //build array t2 as compilation of compTrades (open and historical)
   IntAddIntArr(t2, t3);
   IntAddIntArr(t2, t5);
   int countCompTrades = IntAddIntArr(t2, t6);
   
 //filter out trades which already had a comp trade which is closed and yielded profit > 0
 //i.e. compensated the loss of the orig trade OR which has a comp Trade currently open
   for(int i=0; i < countCompTrades; i++)
   {
      int compTicket = t2[i];
      int origTicket = GetOriginalTicketComp(compTicket);
      double profit = GetTradesProfitInPoints(compTicket);
      
      if(origTicket > 0)
      if(IsOrderOpen(compTicket) || profit > 0 )
         count = IntDelValue(t7, origTicket);
   }
   
   //IntGetLogStr(int arr[], string header = "contents of integer array\n")
   //t1 now holds only trades which did not have a comp Trade
   for(i = 0; i < count; i++)
   {
      int ticket = t7[i];
      
      if(GetTradesProfitInPoints(ticket) < 0)
      {
         Print("found ticket T#"+ticket," which needs a compTrade");
         
         double sl = 0;
         if(UseStopLoss)
            sl = StopLoss;
         double tp = 0;
         if(UseTakeProfit)
            tp = TakeProfit;
            
         OpenCompensatoryTrade(cmd, Lots, MagicNumberCompensatory, sl, sl, ticket);
      }
   }
}

void OpenStaticKompTrade_x(int cmd)
{
   int t1[], t2[], t3[], t4[], t5[], t6[], t7[];
   //get trades in past which have been closed after StartTime
   FillArrWithTickets(t1, MagicNumber, Symbol(), cmd, MODE_TRADES);
   FillArrWithTickets(t2, MagicNumberCompensatory, Symbol(), cmd, MODE_TRADES);
   FillArrWithTickets(t3, MagicNumberCompensatoryM1, Symbol(), cmd, MODE_TRADES);
   FillArrWithTickets(t4, MagicNumber, Symbol(), cmd, MODE_HISTORY);
   FillArrWithTickets(t5, MagicNumberCompensatory, Symbol(), cmd, MODE_HISTORY);
   FillArrWithTickets(t6, MagicNumberCompensatoryM1, Symbol(), cmd, MODE_HISTORY);
   
   IntAddIntArr(t1, t2);
   IntAddIntArr(t1, t3);
   IntAddIntArr(t1, t4);
   IntAddIntArr(t1, t5);
   int count = IntAddIntArr(t1, t6);
   
   count = GetTradesClosedSince(t1, t7, EaStartTime);
   
   int compTrades[], ticketsWithCompTrade[];
   
   IntAddIntArr(t2, t3);
   IntAddIntArr(t2, t5);
   int countCompTrades = IntAddIntArr(t2, t6);
   
   for(int i=0; i < countCompTrades; i++)
   {
      int compTicket = t2[i];
      int origTicket = GetOriginalTicketComp(compTicket);
      double profit = GetTradesProfitInPoints(compTicket);
      
      if(origTicket > 0)
      if(IsOrderOpen(compTicket) || profit > 0)
         count = IntDelValue(t7, origTicket);
   }
   
   //IntGetLogStr(int arr[], string header = "contents of integer array\n")
   //t7 now holds only trades which did not have a comp Trade
   for(i = 0; i < count; i++)
   {
      int ticket = t7[i];
      
      if(GetTradesProfitInPoints(ticket) < 0)
      {
         Print("found ticket T#"+ticket," which needs a compTrade");
         
         double sl = 0;
         if(UseStopLoss)
            sl = StopLoss;
         double tp = 0;
         if(UseTakeProfit)
            tp = TakeProfit;
            
         OpenCompensatoryTrade(cmd, Lots, MagicNumberCompensatory, sl, sl, ticket);
      }
   }
}

/*
   analyzes the OrderComment of the given compTicket and 
   returns the corrsponding original ticket
*/
int GetOriginalTicketComp(int compTicket)
{
   int remeberedTicket = OrderTicket();
   
   int originalTicket = -1;

   if(OrderSelect(compTicket, SELECT_BY_TICKET))
   {
      string comment = OrderComment();
      
      //Print("ordercoment:" + comment);
      
      originalTicket = StrToInteger(StringSubstr(comment, StringFind(comment, "C#")+2));
      
      //Print("found originalTicket="+originalTicket);
   }
   
   if(remeberedTicket > 0)
      OrderSelect(remeberedTicket, SELECT_BY_TICKET);
   
   return(originalTicket);
}