
double GetOpenPriceForCmd(string symbol, int cmd)
{
   string f = "GetOpenPriceForCmd";
   double oPrice = -1.0;

   if(GetGeneralDirForCmd(cmd) == OP_BUY) 
      oPrice = MarketInfo(symbol, MODE_ASK);
   else if(GetGeneralDirForCmd(cmd) == OP_SELL) 
      oPrice = MarketInfo(symbol, MODE_BID);
   /*else
      Error(f, StringConcatenate("couldn\'t determine openPrice for cmd \'",cmd,"\'"));*/
   
   return(oPrice);
}

double GetClosePriceForCmd(string symbol, int cmd)
{
   string f = "GetOpenPriceForCmd";
   double oPrice = -1.0;
   
   if(GetGeneralDirForCmd(cmd) == OP_BUY) 
      oPrice = MarketInfo(symbol, MODE_BID);
   else if(GetGeneralDirForCmd(cmd) == OP_SELL) 
      oPrice = MarketInfo(symbol, MODE_ASK);
   /*else
      Error(f, StringConcatenate("couldn\'t determine closePrice for cmd \'",cmd,"\'"));*/
   
   return(oPrice);
}

int ReverseCmd(int cmd)
{
   int revCmd = -1;
   
   switch(cmd)
   {
      case OP_BUY:
      case OP_SELL:
         cmd = MathAbs(cmd-1);
         break;
      case OP_BUYLIMIT:
         cmd = OP_SELLLIMIT;
         break;
      case OP_SELLLIMIT:
         cmd = OP_BUYLIMIT;
         break;
      case OP_BUYSTOP:
         cmd = OP_SELLSTOP;
         break;
      case OP_SELLSTOP:
         cmd = OP_BUYSTOP;
         break;
   }
   
   return(cmd);
}

int GetGeneralDirForCmd(int cmd)
{
   string f = "GetGeneralDirForCmd";
   int generalDir = -1;
   
   if(cmd == OP_BUY || cmd == OP_BUYLIMIT || cmd == OP_BUYSTOP)
	  generalDir = OP_BUY;
   else if(cmd == OP_SELL || cmd == OP_SELLLIMIT || cmd == OP_SELLSTOP)
	  generalDir = OP_SELL;
   /*else
      Error(f, StringConcatenate("couldn\'t determine general direction for cmd \'",cmd,"\'"));*/
	
	return(generalDir);
}

//+------------------------------------------------------------------+
//|   returns human readable string of given command like OP_BUY for 0
//+------------------------------------------------------------------+
string GetStrFromCommand(int cmd)
{
   string f = "GetStrFromCommand";
   
   switch(cmd)
   {
      case OP_BUY:
         return("OP_BUY");
      case OP_BUYSTOP:
         return("OP_BUYSTOP");
      case OP_BUYLIMIT:
         return("OP_BUYLIMIT");
      case OP_SELL:
         return("OP_SELL");
      case OP_SELLLIMIT:
         return("OP_SELLLIMIT");
      case OP_SELLSTOP:
         return("OP_SELLSTOP");
      default:
         return("NULL");
   }
}

bool CmdIsPending(int cmd)
{
   return((cmd == OP_SELLLIMIT || cmd == OP_SELLSTOP || cmd == OP_BUYSTOP || cmd == OP_BUYLIMIT));
}

double GetSLForCmd(string sym, int cmd, int stop, double p = -1)
{
   string f = StringConcatenate("GetSLForCmd(sym=",sym,", cmd=",cmd,", stop=",stop);
   double sl = 0;
   
   if(stop == 0)
      return(0);
   
   double openPrice;
   if(p != -1)
      openPrice = p;
   else
      openPrice= GetOpenPriceForCmd(sym, cmd);
      
   double point = MarketInfo(sym, MODE_POINT);
   
   if(GetGeneralDirForCmd(cmd) == OP_BUY)
      sl = openPrice - stop*point;
   else if(GetGeneralDirForCmd(cmd) == OP_SELL)
      sl = openPrice + stop*point;
      
   Info(f, StringConcatenate("determined openPrice=",openPrice," so sl=",sl));
      
   return(sl);
}

double GetTPForCmd(string sym, int cmd, int takeProfit, double p = -1)
{
   string f = StringConcatenate("GetTPForCmd(sym=",sym,", cmd=",cmd,", takeProfit=",takeProfit);
   double tp = 0;
   
   if(takeProfit == 0)
      return(0);
   
   double openPrice;
   if(p != -1)
      openPrice = p;
   else
      openPrice= GetOpenPriceForCmd(sym, cmd);
      
   double point = MarketInfo(sym, MODE_POINT);
   
   if(GetGeneralDirForCmd(cmd) == OP_BUY)
      tp = openPrice + takeProfit*point;
   else if(GetGeneralDirForCmd(cmd) == OP_SELL)
      tp = openPrice - takeProfit*point;
      
   Info(f, StringConcatenate("determined openPrice=",openPrice," so tp=",tp));
      
   return(tp);
}