#define DELIMITER_CSV ";"
#define CNT_FIELDS 5

/*
   CSV-Export
      bruchte ich von Ihnen bitte ein Export / Datenreihen jeweils von
         - dem ersten Einstiegskurs zum Phasen-Beginn
         - dem AvgTP Wert zum ersten Einstiegskurs bei Phasen-Beginn
         - dem letzten Einstiegskurs der Phase (d.h. den Einstiegskurs vom letzten offenen Trades

      dann wird dessen Erffnungskurs geschrieben
      -> einfach per GetOpenPriceForCmd() oder so...
      
      genauso die aktuelle PhasenDirection
      -> GetStrFromCommand(CurPhaseDir)
      
      
      AvgTp der brig gebliebenen Trades der letzten Phase, d.h einfach Reverse(aktuellePhase)
      Einstieg des zuletzt geffneten Trades, der zu diese Zeit noch offen ist
      -> 
      
   Dateiname
      FX-MX-Trader_<SYM>_<PERIOD>.csv
*/
void ExportToCsv(int ticket)
{
   if(IsFirstTradeInPhase(ticket))
   {
    //Current open price
      double curOpenPrice = GetOpenPriceForCmd(Symbol(), CurPhase);
      string phaseStr = GetStrFromCommand(CurPhase);

    //Phase direction
      int dir = CurPhase;
      
    //Average TP
      int tickets[];
      int cnt = FillArrWithTickets(tickets, MagicNumber, Symbol(), dir);
      IntDelValue(tickets, ticket);
      double avgOpenPrice = GetAverageOpenPriceForArr(tickets);
      
      double lastAvgTp = 0;
      
      if(avgOpenPrice > 0)
         lastAvgTp = AddAccordingToDir(dir, avgOpenPrice, ProfitPointsAvgTp*Point);
      
    //date
      string date = TimeToStr(TimeCurrent(), TIME_DATE|TIME_MINUTES);
      
    //last Entry last phase
      double lastEntryPriceLastPhase = 0;
      //und letzter Einstiegskurs der Phase
      int lastOrder = GetLastOpenedTicket(MagicNumber, Symbol(), dir, MODE_TRADES, ticket);
      if(lastOrder > 0)
         lastEntryPriceLastPhase = GetTradesOpenPrice(lastOrder);
      
    //writing
      Print("exporting: date: "+date+" phase: "+phaseStr+" curOpenPrice: "+curOpenPrice+" lastEntry: "+
         lastEntryPriceLastPhase+" lastAvgTp: "+lastAvgTp);
         
      string data[1][CNT_FIELDS];
      data[0][0] = date;
      data[0][1] = MyDoubleToStr(curOpenPrice, Digits, ",");
      data[0][2] = phaseStr;
      data[0][3] = MyDoubleToStr(lastAvgTp, Digits, ",");
      data[0][4] = MyDoubleToStr(lastEntryPriceLastPhase, Digits, ",");
      
      Str2CSV(data, GetCSVFileName(), true, 0, WHOLE_ARRAY);
   }
}

string GetCSVFileName()
{
   return("FX-MX-Trader_"+Symbol()+"_"+Period()+".csv");
}

void InitCSVFile(string fileName)
{
   string f = "InitLogFile(fileName="+fileName+")";
   
   if(!FileExists(fileName))
   {
      Info(f, "file does not exist, writing Header...");
      
      string header[1][CNT_FIELDS];
      header[0][0] = "openDate";
      header[0][1] = "firstEntryPriceOfCurrentPhase";
      header[0][2] = "phaseDirection";
      header[0][3] = "avgTpPriceForSameDirectionTrades";
      header[0][4] = "openPriceOfLastSameDirectionTrade";
      
      Str2CSV(header, fileName, false, 0, WHOLE_ARRAY);
   }
   else
      Info(f, "file exist, size: "+MyGetFileSize(fileName));
}

//+------------------------------------------------------------------+
//| Function..: Str2CSV                                     |
//| Parameters: A      - Name of the string array.                   |
//|             sFile  - Output file name, defaults to "My.CSV".     |
//|             bAdd   - If set to 1 (true) appends the data to the  |
//|                      existing file <sFile>, otherwise if set to  |
//|                      the default 0 (false) the previous contents |
//|                      are over written with the new data.         |
//|             iStart - First row to be written to the file, the    |
//|                      default is row 0.                           |
//|             iCount - Number of rows to be written to the file,   |
//|                      the  default is all rows.                   |
//| Purpose...: Write a two dimensional array to a CSV file for      |
//|             viewing with a text editor or further manipulations  |
//|             using a spreadsheet. Usefull for debugging.          |
//| Returns...: Number of rows written, or a negative value if an    |
//|             error occurs.                                        |
//| Sample....: string sProfit[2][2]={"Jan","-150","Feb","200"};     |
//|             StringArrayWrite(sProfit);                           |
//+------------------------------------------------------------------+
int Str2CSV(string A[][], string sFile = "My.CSV", bool bAdd = false, 
                     int iStart = 0, int iCount = WHOLE_ARRAY) 
  {
   /*Info("Str2CSV", "before writing, current (outdated) contents of CSV file:");
   Info("Str2CSV", GetCSVFileStr(sFile));*/
   
   int wMode;
 //if we want to append the content to the file, we have to use FILE_READ also
   if(bAdd)
      wMode = FILE_CSV|FILE_READ|FILE_WRITE;
   else
      wMode = FILE_CSV|FILE_WRITE;
      
 //the file will create it even though touching is not specified, since mode = WRITE
   int handle = MyFileOpen(sFile, wMode, DELIMITER_CSV, 10, 2, false);
   
   if(handle < 0)
   {
      //Error("Str2CSV", "cannot write InfoFile, for reason: see above");
      return;
   }
   
    int iRows = ArrayRange(A, 0), iCols, iRowsWritten, iRow, iCol;
//----
    if(iRows == 0)
    {
      /*Info("Str2CSV", "after writing, new contents of CSV File:");
      Info("Str2CSV", GetCSVFileStr(GetPathToPosInfoFile()));*/
      //Warn("Str2CSV", "given Array empty, leaving file empty!");
      FileClose(handle);
        return (0);
    }
    iCols = ArraySize(A) / iRows;
//----
    if(iCount == WHOLE_ARRAY) 
        iCount = iRows; 
    else 
        iCount = iStart + iCount - 1;
//----
    if(bAdd) 
        FileSeek(handle, 0, SEEK_END);
//----
    for(iRow = iStart; iRow < iCount; iRow++) 
      {
        for(iCol = 0; iCol < iCols-1; iCol++) 
          {
            FileWrite(handle, A[iRow][iCol] + DELIMITER_CSV);
            FileSeek(handle, -2, SEEK_END);
          } 
        FileWrite(handle, A[iRow][iCol]);
        iRowsWritten++;
      }
    FileClose(handle);
    
   /*Info("Str2CSV", "after writing, new contents of CSV File:");
   Info("Str2CSV", GetCSVFileStr(GetPathToPosInfoFile()));*/
   
    return(iRowsWritten);
}