
//+------------------------------------------------------------------+
//|   returns the given file's size in Bytes, on error -1
//+------------------------------------------------------------------+
int MyGetFileSize(string fileName)
{
   string f = "GetFileSize";
   
   int fileHandle = FileOpen(fileName, FILE_READ);   // open the file
   
   if(fileHandle < 0)   // if the file couldn't be opened...
   {
      Error(f, "error while opening file \'"+fileName+"'");
      return(-1);
   }
   
   int size = FileSize(fileHandle);
   
   FileClose(fileHandle);
   
   return(size);
}

//+------------------------------------------------------------------+
//|   opens given file with retry and touch feature, returns handle
//+------------------------------------------------------------------+
//|   WARNING: 
//|      function must be put in here, since FileHandles 
//|      cannot be passed from one module (lib) to another :/
//+------------------------------------------------------------------+
//|   inputs:
//|      string functioName   name of the function MyFileOpen() is called from, for betterlogging
//|      string fileName      path & fileName for file to open
//|      int mode             mode for FileOpen()
//|      string delimiter     delimiter for file
//|      int tries            number of tries for opening the file
//|      int interval         sleepTime between opening tries in sec
//|      bool touch           if touching is enabled / disabled if file not found
//+------------------------------------------------------------------+
//|   return:
//|      int   returns handle to opened file
//+------------------------------------------------------------------+
int MyFileOpen(string fileName, int mode, string delimiter, int tries, 
   int intervalSec, bool touch = false)
{
   string f = "MyFileOpen";
   
   Debug(f, StringConcatenate("started with inputs:",
      "\n  fileName=",fileName,"\n  mode=",mode,"\n  delimiter=",delimiter,
      "\n  tries=",tries,"\n  interval (sec)=",intervalSec,
      "\n  touchingEnabled=",touch));
      
 //prepares the Writing to the file, important is the DELIMITER
   int handle = -1;
   int try = 1;
   
   while(true)
   {
    //this must stay here since if you put it into the while-args then this loop is terminated on
    //isStopped() == TRUE BUT the function is continued, therefore - if touching is enabled - 
    //the file is DELETED!
    //should be placed before the FileOpen()-Command since otherwise this function returns -1
    //having opened the file and other functions calling this one probably won't close the file
    //if they're getting -1 back
    //it's only exited if we're retrying, i.e. the first try is granted even with isStopped=true
    //the reason for this is the following: functions which are calling 
      if(IsStopped() && try > 1)
      {
         Info(f, "tradingStation shutting down, IsStopped=true, returning -1");
         return(-1);
      }
         
      handle = FileOpen(fileName, mode, delimiter);
      
    //error log
      if(handle < 0)
         Warn(f, StringConcatenate("Error while trying to open file: ",fileName,", try(",try,"/",tries,")"));
      else
      {
       //if we already tried and now it worked
         if(try > 1)
            Warn(f, StringConcatenate("successfully opened file: ",fileName,
               " on try(",try,"/",tries,"), fileSize: ",FileSize(handle)));
         else
            Info(f, StringConcatenate("successfully opened file: ",fileName,
               " on try(",try,"/",tries,"), fileSize: ",FileSize(handle)));
            
         return(handle);
      }
      
    //we don't want to increment and Sleep on last iteration, instead we exit from loop
      if(try >= tries)
         break;
         
      Sleep(intervalSec*1000);
      try++;
   }
 //if we arrive here, opening failed after retries, if touching specified, do it!
   if(touch)
   {
    //since this is a very dangerous action we want to check a last time after the loop
      handle = FileOpen(fileName, mode, delimiter);
      if(handle > -1)
      {
         Warn(f, "stragely we entered the touchinLogic but now I _did_ find the file, exiting, returning handle");
         return(handle);
      }
      
    //OK - This is when we really open the file for writing, i.e. we empty / delete it if existing
      handle = FileOpen(fileName, FILE_WRITE, delimiter);
      
      if(handle < 0)
         Error(f, StringConcatenate("file ",fileName," couldn\'t be found after ",try," tries, touching enabled, but couldn't create it!"));
      else
         Info(f, StringConcatenate("file ",fileName," couldn\'t be found after ",try," tries, touching enabled, successfully created it"));
   }
   else
      Error(f, StringConcatenate("couldn\'t open file ",fileName," after ",try," tries, Touching was disabled?"));
      
   return(handle);
}

bool FileExists(string fileName)
{
   string f = "FileExists";
   
   Debug(f, StringConcatenate("started with inputs:\n  fileName=",fileName));
   
   int handle = FileOpen(fileName, FILE_READ);
   
   int lastErr = GetLastError();
   
   Debug(f, StringConcatenate("opened file, lastErr = ",lastErr));
   
 //if the lastError is not what is expected then do loggin
   if(lastErr != 0 && lastErr != ERR_CANNOT_OPEN_FILE)
      Warn(f, StringConcatenate("while opening file ",fileName,
         " error occurred,",ErrGetLogStr(lastErr)));
   
   if(handle >= 0)
   {
      FileClose(handle);
      return(true);
   }
   else
      return(false);
}