
/*
   Bei jedem M1-Phasenwechsel sammle alle Trades aus den Phasen vorher
   fr jeden dieser Trades:
      berechne Abstand (aktueller Markt zu damaligem Einstiegspreis)
      wenn dieser Abstand > OpenHedge-Variable:
         Hedge Order
*/
void OpenHedgeOnPhaseChange_x(int curPhase_x)
{
   int openTickets[], openTicketsFromOldPhase[];
   FillArrWithTickets(openTickets, MagicNumber, Symbol(), ReverseCmd(curPhase_x), MODE_TRADES);
   int countOpen = GetTradesOpenedBetween(0, CurPhaseStart_x-1, openTickets, openTicketsFromOldPhase);
   
   //if(countOpen>0)
      //Print("running over "+countOpen+" positions..."+curPhase_x);
   
   for(int i=0; i < countOpen; i++)
   {
      int ticket = openTicketsFromOldPhase[i];
      
      if(IsHedged(ticket))
         continue;
         
      double openPrice = GetTradesOpenPrice(ticket);
      int dir = GetTradesCmd(ticket);
      double curPrice = GetClosePriceForCmd(Symbol(), dir);
      
      if(Exceeds(dir, openPrice, curPrice, -OpenM1HedgeAfter*Point))
      {
         HedgeOrder(ticket);
      }
   }
}

/*void CloseHedgeTrades(int curPhase_x)
{   
   int tickets[];
   int cnt = FillArrWithTickets(tickets, MagicNumberHedge, Symbol(), ReverseCmd(curPhase_x));
   
   for(int i=0; i < cnt; i++)
   {
      int ticketH = tickets[i];
      int ticketO = GetOriginalTicket(ticketH);
      
      double openPriceH = GetTradesOpenPrice(ticketH);
      double openPriceO = GetTradesOpenPrice(ticketO);
      double curPrice = GetClosePriceForCmd(Symbol(), curPhase_x);
      
      if(CloseHedgeAtExceeding)
      {
         if(Exceeds(CurPhase_x, curPrice, openPriceO))
            CloseOrder(ticketH);
      }
      else
         CloseOrder(ticketH);
   }
      
}*/

bool IsHedged(int ticket)
{
   int hedgeTicket = GetHedgeTicket(ticket);
   
   bool res = false;
   
   if(hedgeTicket > 0)
      if(IsOrderOpen(hedgeTicket))
         res = true;
   
   return(res);
}

int HedgeOrder(int ticket)
{
   if(IsHedged(ticket))
      return(GetHedgeTicket(ticket));

   if(!OrderSelect(ticket, SELECT_BY_TICKET))
      return(-1);
   
   int origType = OrderType();
   int cmd = ReverseCmd(GetGeneralDirForCmd(origType));
   double origLots = OrderLots();
   double origSl = OrderStopLoss();
   double origTp = OrderTakeProfit();
   
   color clr;
   if(cmd == OP_BUY)
      clr = CLR_OPEN_HEDGE_LONG;
   else if(cmd == OP_SELL)
      clr = CLR_OPEN_HEDGE_SHORT;

   Print("hedging order T#", ticket);
   
   int hedgeTicket = OrderSend(Symbol(), cmd, origLots, GetOpenPriceForCmd(Symbol(), cmd), 
      Slippage, 0, 0, "H#"+ticket, MagicNumberHedge, 0, clr);
      
   return(hedgeTicket);
}

/*
   iterates over list of open hedge positions and closes all which don't have a corresponding
   normal position
*/
void CloseLonelyHedgePositions()
{
   int tickets[];
   int cnt = FillArrWithTickets(tickets, MagicNumberHedge, Symbol(), MODE_ALL);
   
   for(int i=0; i < cnt; i++)
   {
      int ticket = tickets[i];
      
      int origTicket = GetOriginalTicket(ticket);
      
      if(!IsOrderOpen(origTicket))
      {
         Print("closing lonely hedge position T#"+ticket+" which used to hedge origTicket#"+origTicket);
         
         CloseOrder(ticket);
      }
   }
}

// ***********************************************************************************
//                 V. 7)  Hedging
// *********************************************************************************** 

void TickHedging()
{
   //arrays: h_positions[], h_hedgedAt[], h_highestPrice[], h_lowestPrice[]

   //fr jede offene Market position
   for(int i=0; i < OrdersTotal(); i++)
   {
      if(!OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
         continue;
         
      if(OrderMagicNumber() != MagicNumber)
         continue;
         
      int origTicket = OrderTicket();
      int origType = OrderType();
      double origLots = OrderLots();
      double origOpenPrice = OrderOpenPrice();
      double origSl = OrderStopLoss();
      double origTp = OrderTakeProfit();
            
      int hedgeTicket = GetHedgeTicket(origTicket);
   
      if(hedgeTicket > 0)
         AlignSlTp(origTicket, hedgeTicket);
      
      //suche indexposition aus positions[]
      int idx = IntGetIndex(h_positions, origTicket);
      
      if(idx < 0)
      {
         idx = IntAdd(h_positions, origTicket);
         DoubleAdd(h_highestPrice, origOpenPrice);
         DoubleAdd(h_lowestPrice, 0);
         DoubleAdd(h_hedgedAt, -1);
         
         idx --;
      }
      
      //wenn Kurs jenseits von Max ist, ndere max
      double p = Close[0];
      if(Exceeds(origType, p, h_highestPrice[idx]))
         h_highestPrice[idx] = p;
   
      //wenn hedgedAt[] <= 0, also noch nicht gehedged:
      if(h_hedgedAt[idx] <= 0)
      {
         double retracement;
         
         if(origType == OP_BUY)
            retracement = h_highestPrice[idx] - p;
         else if(origType == OP_SELL)
            retracement = p - h_highestPrice[idx];
            
         retracement = Point2Pip(retracement/Point);
         
         //wenn aktueller Kurs gleich oder mehr als HedgeAtRetracementPips von Max entfernt
         if(retracement >= HedgeAtRetracementPips)
         {
            //dann hedge und setze HedgedAt auf openKurs
            Print("hedging order T#",origTicket," at: ",p,". h_highestPrice=",h_highestPrice[idx]);
            h_hedgedAt[idx] = p;
                  
            int cmd = ReverseCmd(origType);
            
            color clr;
            if(cmd == OP_BUY)
               clr = CLR_OPEN_HEDGE_LONG;
            else if(cmd == OP_SELL)
               clr = CLR_OPEN_HEDGE_SHORT;

            hedgeTicket = OrderSend(Symbol(), cmd, origLots, GetOpenPriceForCmd(Symbol(), cmd), 
               Slippage, origSl, origTp, "H#"+origTicket, MagicNumberHedge, 0, clr);
            
            //Print("hedge ticket: T#",GetHedgeTicket(origTicket));
         }
      }
      //wenn nicht, also schon gehedged:
      else
      {
         //wenn Kurs jenseits von Max ist, ndere max
         if(Exceeds(origType, h_lowestPrice[idx], p) || h_lowestPrice[idx] == 0)
            h_lowestPrice[idx] = p;
         
         double back;
         
         if(origType == OP_BUY)
            back = p - h_lowestPrice[idx];
         else if(origType == OP_SELL)
            back = h_lowestPrice[idx] - p;
            
         back = Point2Pip(back/Point);
         
         //wenn aktueller Kurs gleich oder mehr als CloseHedgeAtPips
         if(back >= CloseHedgeAtPips)
         {
            //schliee hedge, setze hedgedAt = 0;
            Print("closing hedge for order T#",origTicket,". Close=",p,
               " h_lowestPrice[idx]=",h_lowestPrice[idx]," back=",back);
               
            h_hedgedAt[idx] = 0;
            h_lowestPrice[idx] = 0;
            h_highestPrice[idx] = p;
            
            clr = Yellow;

            if(OrderSelect(hedgeTicket, SELECT_BY_TICKET))
               OrderClose(hedgeTicket, OrderLots(), 
                  GetClosePriceForCmd(Symbol(), OrderType()), Slippage, clr);
         }
      }
   }
}

void AlignSlTp(int ticket1, int ticket2)
{
   int remeberedTicket = OrderTicket();

   if(!OrderSelect(ticket1, SELECT_BY_TICKET))
      return;

   double sl1 = OrderStopLoss();
   double tp1 = OrderTakeProfit();
   
   if(!OrderSelect(ticket2, SELECT_BY_TICKET))
      return;
      
   double sl2 = OrderStopLoss();
   double tp2 = OrderTakeProfit();
   
   if(sl1 != sl2 || tp1 != tp2)
      OrderModify(ticket2, OrderOpenPrice(), sl1, tp1, OrderExpiration());
      
   if(remeberedTicket > 0)
      OrderSelect(remeberedTicket, SELECT_BY_TICKET);
}

int GetHedgeTicket(int normalTicket)
{
   int remeberedTicket = OrderTicket();

   int hedgeTicket = -1;
   
   for(int i=0; i < OrdersTotal(); i++)
   {
      if(!OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
         continue;
         
      /*if(OrderMagicNumber() != MagicNumberHedge)
         continue;*/
      
      string comment = OrderComment();
      
      if(StringFind(comment, "H#"+normalTicket) > -1)
      {
         hedgeTicket = OrderTicket();
         break;
      }
   }
   
   if(remeberedTicket > 0)
      OrderSelect(remeberedTicket, SELECT_BY_TICKET);
   
   return(hedgeTicket);
}

/*
   analyzes the OrderComment of the given hedgeTicket and 
   returns the corrsponding original ticket
*/
int GetOriginalTicket(int hedgeTicket)
{
   int remeberedTicket = OrderTicket();
   
   int originalTicket = -1;

   if(OrderSelect(hedgeTicket, SELECT_BY_TICKET))
   {
      string comment = OrderComment();
      
      //Print("ordercoment:" + comment);
      
      originalTicket = StrToInteger(StringSubstr(comment, StringFind(comment, "H#")+2));
      
      //Print("found originalTicket="+originalTicket);
   }
   
   if(remeberedTicket > 0)
      OrderSelect(remeberedTicket, SELECT_BY_TICKET);
   
   return(originalTicket);
}