#define ROUND_NORMAL 0
#define ROUND_FLOOR 1
#define ROUND_CEIL 2

double RoundLots(double lots, int roundMode = ROUND_NORMAL)
{
   string f = StringConcatenate("RoundLots(lots=",lots,", roundMode=",roundMode,")");
   
   Info(f, StringConcatenate("determined lots before rounding: ",lots));
   
   double minLot = MarketInfo(Symbol(), MODE_MINLOT);
   double lotStep = MarketInfo(Symbol(), MODE_LOTSTEP);
   
   Info(f, StringConcatenate("minLot=",minLot,", lotStep=",lotStep));
   
   if(lots <= 0)
   {
      Info(f, "about to round lots <= 0 ("+lots+"), returning this value");
   }
   else if(lots < minLot)
      lots = minLot;
   else
   {
    //normalizing absolutely necessary at this place!
      double lotSteps = NormalizeDouble(lots / lotStep, 12);
      
      if(roundMode == ROUND_FLOOR)
         lotSteps = MathFloor(lotSteps);
      else if(roundMode == ROUND_CEIL)
         lotSteps = MathCeil(lotSteps);
      else if(roundMode == ROUND_NORMAL)
         lotSteps = MathRound(lotSteps);
      else
      {
         Error(f, StringConcatenate("invalid roundMode \'",roundMode,"\', exiting."));
         
         return(-1);
      }
      
      Info(f, StringConcatenate("lotSteps=",lotSteps,":",MathFloor( lots / lotStep )));
         
      lots = lotSteps * lotStep;
   }
   
   Info(f, StringConcatenate("determined lots after successful rounding: ",lots));
   
   return(lots);
}