
#define OBJ_PREFIX "FX-MX_"
#define OBJ_NAME_AVG_TP "AvgTp"
#define OBJ_NAME_PHASE_LONG "longPhase"
#define OBJ_NAME_PHASE_SHORT "shortPhase"
#define OBJ_NAME_PHASE_LONG_X "longPhase_x"
#define OBJ_NAME_PHASE_SHORT_X "shortPhase_x"
#define OBJ_NAME_PHASE_NONSENSE_X "nonsensePhase_x"
#define OBJ_NAME_PHASE_LONG "longPhase"
#define OBJ_NAME_PHASE_SHORT "shortPhase"
#define OBJ_NAME_PHASE_NONSENSE "nonsensePhase"

void DrawPhaseLine(int dir)
{
   string name = OBJ_PREFIX;
   color clr;
   
   if(dir == OP_BUY)
   {
      name = name+OBJ_NAME_PHASE_LONG;
      clr = Blue;
   }
   else if(dir == OP_SELL)
   {
      name = name+OBJ_NAME_PHASE_SHORT;
      clr = Red;
   }
   else
   {
      name = name+OBJ_NAME_PHASE_NONSENSE;
      clr = White;
   }
      
   name = name+TimeCurrent();
      
   CreateVLine(name, TimeCurrent(), clr, 2, STYLE_DOT, "", true);
}

void DrawPhaseArrow(int dir)
{
   string name = OBJ_PREFIX;
   color clr;
   int arrowCode;
   double price;
   
   if(dir == OP_BUY)
   {
      name = name+OBJ_NAME_PHASE_LONG_X;
      clr = Blue;
      arrowCode = 241;
      price = iLow(Symbol(), PERIOD_M1, 0)-5*Point;
   }
   else if(dir == OP_SELL)
   {
      name = name+OBJ_NAME_PHASE_SHORT_X;
      clr = Red;
      arrowCode = 242;
      price = iHigh(Symbol(), PERIOD_M1, 0)+5*Point;
   }
   else
   {
      name = name+OBJ_NAME_PHASE_NONSENSE_X;
      clr = White;
      arrowCode = 251;
      price = iHigh(Symbol(), PERIOD_M1, 0)+5*Point;
   }
      
   name = name+TimeCurrent();
      
   CreateArrow(name, TimeCurrent(), price, arrowCode, clr, 3, "", false, OBJ_PERIOD_M1);
}


void ShowAvgTp(int dir, double p, bool hedgeAvgTp=false)
{
   int style = STYLE_DASHDOT;
   if(hedgeAvgTp)
      style = STYLE_DOT;
      
   CreateHLine(GetNameForAvgTPLine(dir, hedgeAvgTp), p, GetAvgTpLineColor(dir), 1, style);
}

void HideAvgTp(int dir, bool hedgeAvgTp=false)
{
   ObjectDelete(GetNameForAvgTPLine(dir, hedgeAvgTp));
}

color GetAvgTpLineColor(int dir)
{
   color clr = White;
   
   if(dir == OP_BUY)
      clr = PaleGreen;
   if(dir == OP_SELL)
      clr = OrangeRed;
      
   return(clr);
}

string GetNameForAvgTPLine(int dir, bool hedgeAvgTp)
{
   string s = OBJ_PREFIX+OBJ_NAME_AVG_TP+dir;
   
   if(hedgeAvgTp)
      s = s+"_H";
   
   return(s);
}

void DeleteAllObjects()
{
   DeleteObjectsContaining(OBJ_PREFIX);
}

void DeleteObjectsContaining(string match)
{
   int obj_total = ObjectsTotal();
   
   string objName;
 
 //it has to be iterated _up_ the list like when deleting orders 
   for(int i = (obj_total-1); i >= 0; i--)
   {
      objName = ObjectName(i);
      
    //if the current object's name contains match
      if(StringFind(objName, match) > -1 )
         ObjectDelete(objName);
   }
   
 //we gotta redraw now
   WindowRedraw();
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CreateHLine(string name, double p1, color col=White, int width=1, int style=STYLE_DOT, 
                 string description="", bool background=false, bool overwrite=true)
{
   if(ObjectFind(name) >= 0)
   {
      if(overwrite)
      {
         ObjectDelete(name);
         ObjectCreate(name, OBJ_HLINE, 0, 0, p1);
      }
   }
   else
      ObjectCreate(name, OBJ_HLINE, 0, 0, p1);

   ObjectSet(name, OBJPROP_COLOR	, col);
   ObjectSet(name, OBJPROP_STYLE	, style);
   ObjectSet(name, OBJPROP_WIDTH	, width);
   ObjectSet(name, OBJPROP_BACK	, background);
   ObjectSetText(name, description);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CreateVLine(string name, datetime t1, color col=White, int width=1, int style=STYLE_DOT, 
                 string description="", bool background=false, bool overwrite=true)
{
   if(ObjectFind(name) >= 0)
   {
      if(overwrite)
      {
         ObjectDelete(name);
         ObjectCreate(name, OBJ_VLINE, 0, t1, 0);
      }
   }
   else
      ObjectCreate(name, OBJ_VLINE, 0, t1, 0);

   ObjectSet(name, OBJPROP_COLOR	, col);
   ObjectSet(name, OBJPROP_STYLE	, style);
   ObjectSet(name, OBJPROP_WIDTH	, width);
   ObjectSet(name, OBJPROP_BACK	, background);
   ObjectSetText(name, description);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CreateArrow(string name, datetime t1, double p1, int arrowCode, color col=White, int width=1,
                 string description="", bool background=false, int showOnTFs=OBJ_ALL_PERIODS, bool overwrite=true)
{
   if(ObjectFind(name) >= 0)
   {
      if(overwrite)
      {
         ObjectDelete(name);
         ObjectCreate(name, OBJ_ARROW, 0, t1, p1);
      }
   }
   else
      ObjectCreate(name, OBJ_ARROW, 0, t1, p1);

   ObjectSet(name, OBJPROP_ARROWCODE, arrowCode);
   ObjectSet(name, OBJPROP_COLOR	, col);
   ObjectSet(name, OBJPROP_WIDTH	, width);
   ObjectSet(name, OBJPROP_BACK	, background);
   ObjectSet(name, OBJPROP_TIMEFRAMES, showOnTFs);
   ObjectSetText(name, description);
}

