
#define MODE_POINTS 1
#define MODE_ACCOUNT_CURRENCY 2

int GetTradesClosedSince(int tickets[], int& resultTickets[], datetime time)
{
   string f = "GetTradesClosedAfter(time="+TimeToStr(time)+")";
   
   ArrayResize(resultTickets, 0);
   
   int size = ArraySize(tickets);
   
   //Info(f, "started");
   
   for(int i=0; i < size; i++)
   {
      int ticket = tickets[i];
      
      if(!OrderSelect(ticket, SELECT_BY_TICKET))
         continue;
         
      datetime curOrdersCloseDate = OrderCloseTime();
         
      if(curOrdersCloseDate >= time)
      {
         IntAdd(resultTickets, ticket);
         //Info(f, "adding ticket T#"+ticket+" to array");
      }
   }
   
   int resultSize = ArraySize(resultTickets);
   
   //Info(f, "exiting with resultSize="+resultSize);
   
   return(resultSize);
}

int GetTradesOpenedBetween(datetime from, datetime upTo, int tickets[], int& resultTickets[])
{
   string f = "GetTradesClosedBetween(from="+TimeToStr(from)+" upTo="+TimeToStr(upTo)+")";
   
   ArrayResize(resultTickets, 0);
   
   int size = ArraySize(tickets);
   
   Debug(f, "started");
   
   for(int i=0; i < size; i++)
   {
      int ticket = tickets[i];
      
      if(!OrderSelect(ticket, SELECT_BY_TICKET))
         continue;
         
      datetime curOrdersOpenDate = OrderOpenTime();
         
      if(curOrdersOpenDate >= from && curOrdersOpenDate <= upTo)
      {
         IntAdd(resultTickets, ticket);
         Debug(f, "adding ticket T#"+ticket+" to array");
      }
      else
         Debug(f, "not adding ticket T#"+ticket+" closeTime: "+TimeToStr(curOrdersOpenDate));
   }
   
   int resultSize = ArraySize(resultTickets);
   
   Debug(f, "exiting with resultSize="+resultSize);
   
   return(resultSize);
}

double GetTradesOpenPrice(int ticket)
{
   string f = "GetTradesOpenPrice";
   
   if(!OrderSelect(ticket, SELECT_BY_TICKET))
   {
      Warn(f, StringConcatenate("couldn\'t select ticket T#",ticket,", returning -1"));
      return(-1);
   }
   
   double openPrice = NormalizeDouble(OrderOpenPrice(), Digits);
   
   return(openPrice);
}

int GetBiggestLoser(int tickets[])
{
   string f = "GetBiggestLoser";
   
   int size = ArraySize(tickets);
   
   //Info(f, "started");
   
   double lastProfit, curProfit;
   int loserIdx = -1;
   for(int i=0; i < size; i++)
   {
      int ticket = tickets[i];
      
      if(!OrderSelect(ticket, SELECT_BY_TICKET))
         continue;
         
      curProfit = OrderProfit();
         
      if(curProfit < 0 && curProfit < lastProfit)
      {
         loserIdx = i;
         
         lastProfit = curProfit;
      }
   }
   
   //Print(f, "determined loserIdx="+loserIdx);
   
   int loserTicket = -1;
   if(loserIdx >= 0)
      loserTicket = tickets[loserIdx];
      
   return(loserTicket);
}

double GetTradesProfit(int ticket)
{
   string f = "GetTradesProfit";
   
   if(!OrderSelect(ticket, SELECT_BY_TICKET))
   {
      //Warn(f, StringConcatenate("couldn\'t select ticket T#",ticket,", returning -1"));
      return(0);
   }
   
   double profit = (OrderProfit() + OrderSwap() + OrderCommission());
   
   return(profit);
}

/*
   FillArrWithTickets(tickets, MagicNumber, Symbol(), MODE_ALL, MODE_HISTORY);
*/
int GetMostRecentCloseTrade(int tickets[])
{
   string f = "GetMostRecentCloseTrade";
   
   int size = ArraySize(tickets);
   
   //Info(f, "started");
   
   datetime lastOrdersCloseDate, curOrdersCloseDate;
   int mostRecentIdx = -1;
   for(int i=0; i < size; i++)
   {
      int ticket = tickets[i];
      
      if(!OrderSelect(ticket, SELECT_BY_TICKET))
         continue;
         
      curOrdersCloseDate = OrderCloseTime();
         
      if(curOrdersCloseDate > lastOrdersCloseDate)
      {
         mostRecentIdx = i;
      
         lastOrdersCloseDate = curOrdersCloseDate;
      }
   }
   
   //Info(f, "determined mostRecentIdx="+mostRecentIdx);
   
   int mostRecentTicket = -1;
   if(mostRecentIdx >= 0)
      mostRecentTicket = tickets[mostRecentIdx];
   
   return(mostRecentTicket);
}

bool OrderHitTakeProfit(int ticket)
{
   string f = StringConcatenate("OrderHitTakeProfit(",ticket,")");
   
   //Debug(f, "started");
   
   if(!OrderSelect(ticket, SELECT_BY_TICKET))
      return(false);
      
 //for S/LS it's [sl], sometimes [sl/a] or [sl/gap] etc.
   if(StringFind(OrderComment(), "[tp") >= 0)
   {
      //Info(f, StringConcatenate("T#",ticket," hit T/P!"));
      
      return(true);
   }
   else      
   {
      //Debug(f, "hasn\'t hit TP (yet), OComment: "+OrderComment());
      
      return(false);
   }
}

#define ERR_SELECT -1
#define STATUS_OPEN 1
#define STATUS_CLOSED 0

int GetOrderStatus(int ticket)
{
   if(!OrderSelect(ticket, SELECT_BY_TICKET))
      return(ERR_SELECT);
      
   if(OrderCloseTime() > 0)
      return(STATUS_CLOSED);
   else
      return(STATUS_OPEN);
}

bool IsOrderOpen(int ticket)
{
   return(GetOrderStatus(ticket) == STATUS_OPEN);
}

bool IsOrderClosed(int ticket)
{
   return(GetOrderStatus(ticket) == STATUS_CLOSED);
}

bool ModeAllowsCmd(int mode, int cmd)
{
   int genDir = GetGeneralDirForCmd(cmd);
   bool res = false;
   
   if(mode == cmd)
      return(true);
   
   switch(mode)
   {
      case MODE_SHORT:
         res = (genDir == OP_SELL);
         break;
         
      case MODE_LONG:
         res = (genDir == OP_BUY);
         break;
         
      case MODE_PENDING:
         res = (CmdIsPending(cmd));
         break;
         
      case MODE_MARKET:
         res = (!CmdIsPending(cmd));
         break;
         
      case MODE_PENDING_LONG:
         res = (CmdIsPending(cmd) && genDir == OP_BUY);
         break;
         
      case MODE_PENDING_SHORT:
         res = (CmdIsPending(cmd) && genDir == OP_SELL);
         break;
         
      case MODE_MARKET_LONG:
         res = (!CmdIsPending(cmd) && genDir == OP_BUY);
         break;
         
      case MODE_MARKET_SHORT:
         res = (!CmdIsPending(cmd) && genDir == OP_SELL);
         break;
         
      case MODE_ALL:
         res = true;
         break;
   }
   
   return(res);
}

int GetLastOpenedTicket(int magic = -1, string sym = "", int cmd = MODE_ALL, int histMode = MODE_TRADES, int exceptTicket = -1)
{
   string f = "GetLastTicket(sym="+sym+", cmd="+cmd+")";
   int ticket = -1;
   
   int tickets[];
   int ticketCnt = FillArrWithTickets(tickets, magic, sym, cmd, histMode);
   
   if(exceptTicket > 0)
      ticketCnt = IntDelValue(tickets, exceptTicket);
   
   if(ticketCnt <= 0)
      return(ticket);
      
   ArraySort(tickets, WHOLE_ARRAY, 0, MODE_DESCEND);
   
   ticket = tickets[0];
   
   Info(f, "tickets[0]="+ticket);
   
   return(ticket);
}

double GetTradesLots(int ticket)
{
   string f = "GetTradesLots";
   
   if(!OrderSelect(ticket, SELECT_BY_TICKET))
   {
      Warn(f, StringConcatenate("couldn\'t select ticket T#",ticket,", returning -1"));
      return(-1);
   }
   
   double lots = (OrderLots());
   
   return(lots);
}

double GetProfitFromArray(int tickets[], int mode = MODE_POINTS)
{
   return(GetCumulatedProfit(tickets, mode));
}

double GetLotsFromArray(int tickets[])
{
   double lots = 0;
   int cnt = ArraySize(tickets);
   
   for(int i=0; i < cnt; i++)
   {
      int ticket = tickets[i];
      lots += GetTradesLots(ticket);
   }
   
   return(lots);
}

int GetTradesCmd(int ticket)
{
   string f = StringConcatenate("GetTradesCmd(ticket=",ticket,")");
   
   if(!OrderSelect(ticket, SELECT_BY_TICKET))
   {
      Warn(f, StringConcatenate("couldn\'t select ticket T#",ticket,", returning -1"));
      return(0);
   }
   
   int cmd = OrderType();
   
   return(cmd);
}

int FillArrWithTickets(int& tickets[], int magic = -1, string symbol = "", int modeOrCmd = MODE_ALL, int modeHistory = MODE_TRADES)
{
   string f = "FillArrWithMagics(magic="+magic+", sym="+symbol+", mode="+modeOrCmd+")";
   
   //Debug(f, "started");
   
   ArrayResize(tickets, 0);
   
   int oTotal = OrdersTotal();
   
   if(modeHistory == MODE_HISTORY)
      oTotal = OrdersHistoryTotal();
      
   for(int i=0; i < oTotal; i++)
   {
      if(!OrderSelect(i, SELECT_BY_POS, modeHistory))
      {
         //Warn(f, StringConcatenate("error while selecting order #",i,", aborting"));
         continue;
      }
        
      if((OrderMagicNumber() == magic || magic == -1) &&
         (OrderSymbol() == symbol || symbol == "") && ModeAllowsCmd(modeOrCmd, OrderType()))
      {
         //Debug(f, StringConcatenate("adding ticket T#",OrderTicket()," to array of tickets"));
         
         IntAdd(tickets, OrderTicket());
      }
   }
   
   return(ArraySize(tickets));
}

double GetTradesProfitInAccCurr(int ticket)
{
   return(GetTradesProfit(ticket));
}

double GetTradesProfitInPoints(int ticket)
{
   string f = "GetTradesProfitInPoints";
   
   if(!OrderSelect(ticket, SELECT_BY_TICKET))
   {
      Warn(f, StringConcatenate("couldn\'t select ticket T#",ticket,", returning 0"));
      return(0);
   }
   
   double profit = CurOrderGetProfitInPoints();
   
   return(profit);
}

double GetCumulatedProfit(int tickets[], int mode = MODE_POINTS)
{
   string f = "GetCumulatedProfit(mode="+mode+")";
   
   int cnt = ArraySize(tickets);
   double profit = 0;
   
   for(int i=0; i < cnt; i++)
   {
      int ticket = tickets[i];
      
      if(mode == MODE_POINTS)
         profit += GetTradesProfitInPoints(ticket);
      else if(mode == MODE_ACCOUNT_CURRENCY)
         profit += GetTradesProfitInAccCurr(ticket);
   }
   
   Info(f, "determined profit from "+cnt+" trades: "+profit);
   
   return(profit);
}

