/*
   Vorhandene Inputs:
   
   extern string I = "-- Trade Open Restrictions --";
   extern bool RestrictByMaxOpenTradesInPhase = true; 
   extern int  MaxOpenTradesInPhase = 2; 
   extern bool RestrictByMaxOpenTradesOverall = true; 
   extern int  MaxOpenTradesOverall = 5;
   extern bool RestrictByMaxDynLots = true; 
   extern double MaxDynLots = 20;
*/

bool NewTradeRestricted(int cmd)
{
   bool restricted = false;
   
   if(RestrictByMaxOpenTradesInPhase)
   {
      int openCurPhase = GetTradeCountCurPhase(CurPhase, CurPhaseStart);
      
      if(openCurPhase >= MaxOpenTradesInPhase)
      {
         Print("order restricted, ",DoubleToStr(openCurPhase, 0)," open Orders, max allowed: ",DoubleToStr(MaxOpenTradesInPhase, 0));
         restricted = true;
      }
   }
   if(!restricted && RestrictByMaxOpenTradesOverall)
   {
      int openOverall = GetTradeCountOverall();
      
      if(openOverall >= MaxOpenTradesOverall)
      {
         Print("order restricted, ",DoubleToStr(openOverall, 0)," overall open Orders, max allowed: ",DoubleToStr(MaxOpenTradesOverall, 0));
         restricted = true;
      }
   }
   if(!restricted && RestrictByMaxDynLots)
   {
      int loserTicket = -1;
      double potentialDynLots = GetBiggestLoserCompLots(cmd, loserTicket);
      
      if(potentialDynLots >= MaxDynLots)
      {
         Print("order restricted, "+DoubleToStr(potentialDynLots, 2)+" potentially opened dyn lots for T#"+loserTicket+
            ", max allowed: "+DoubleToStr(MaxDynLots, 2));
         restricted = true;
      }
   }
   /*if(!restricted && RestrictByMaxCompTrades)
   {
      int countCompTrades = GetCountCompTradesThisPhase(CurPhase, CurPhaseStart);
      
      if(countCompTrades >= MaxCompTrades)
      {
         Print("order restricted, ",DoubleToStr(countCompTrades, 0)," open Orders, max allowed: ",DoubleToStr(MaxCompTrades, 0));
         restricted = true;
      }
   }*/
   
   return(restricted);
}

bool NewCompTradeRestricted(int cmd)
{
   bool restricted = false;
   
   if(!restricted && RestrictByMaxCompTrades)
   {
      int countCompTrades = GetCountCompTradesThisPhase(CurPhase, CurPhaseStart);
      
      if(countCompTrades >= MaxCompTrades)
      {
         Print("comp trade restricted, ",DoubleToStr(countCompTrades, 0)," open comp Orders, max allowed: ",DoubleToStr(MaxCompTrades, 0));
         restricted = true;
      }
   }
   
   return(restricted);
}

int GetCountCompTradesThisPhase(int curPhase, datetime curPhaseStart)
{   
   int compTrades1[], compTrades2[], compTrades1H[], compTrades2H[], tickets1[], tickets2[], tickets1H[], tickets2H[];
   ArrayResize(compTrades1, 0);
   ArrayResize(compTrades2, 0);
   ArrayResize(tickets1, 0);
   ArrayResize(tickets2, 0);
   
   int compCount = FillArrWithTickets(compTrades1, MagicNumberCompensatoryM1, Symbol(), curPhase, MODE_TRADES);
   int compCount2 = FillArrWithTickets(compTrades2, MagicNumberCompensatory, Symbol(), curPhase, MODE_TRADES);
   int compCountH = FillArrWithTickets(compTrades1H, MagicNumberCompensatoryM1, Symbol(), curPhase, MODE_HISTORY);
   int compCount2H = FillArrWithTickets(compTrades2H, MagicNumberCompensatory, Symbol(), curPhase, MODE_HISTORY);
   
 //tickets wird mit "normalen" Trades der aktuellen PhasenRtg. gefllt
   int ticketCount = GetTradesOpenedBetween(curPhaseStart, TimeCurrent()+1, compTrades1, tickets1);
   ticketCount = GetTradesOpenedBetween(curPhaseStart, TimeCurrent()+1, compTrades2, tickets2);
   ticketCount = GetTradesOpenedBetween(curPhaseStart, TimeCurrent()+1, compTrades1H, tickets1H);
   ticketCount = GetTradesOpenedBetween(curPhaseStart, TimeCurrent()+1, compTrades2H, tickets2H);
   
 //ticktes wird mit HedgeTickets verschnitten
   ticketCount = IntAddIntArr(tickets1, tickets2);
   ticketCount = IntAddIntArr(tickets1, tickets1H);
   ticketCount = IntAddIntArr(tickets1, tickets2H);
   
   return(ticketCount);
}

int GetTradeCountCurPhase(int curPhase, datetime curPhaseStart)
{   
   int tickets[], tempTickets[], ticketsH[], tempTicketsH[];
   ArrayResize(ticketsH, 0);
   ArrayResize(tempTickets, 0);
   ArrayResize(tempTicketsH, 0);
   
   int ticketCount = FillArrWithTickets(tempTickets, MagicNumber, Symbol(), curPhase, MODE_TRADES);
   int ticketCountH = FillArrWithTickets(tempTicketsH, MagicNumberHedge, Symbol(), ReverseCmd(curPhase), MODE_TRADES);
   //int compCount = FillArrWithTickets(compTrades, MagicNumberCompensatoryM1, Symbol(), curPhase, MODE_TRADES);
   
 //tickets wird mit "normalen" Trades der aktuellen PhasenRtg. gefllt
   ticketCount = GetTradesOpenedBetween(curPhaseStart, TimeCurrent()+1, tempTickets, tickets);
   ticketCount = GetTradesOpenedBetween(curPhaseStart, TimeCurrent()+1, tempTicketsH, ticketsH);
 //ticktes wird mit HedgeTickets verschnitten
   ticketCount = IntAddIntArr(tickets, ticketsH);
   
   return(ticketCount);
}

int GetTradeCountOverall()
{   
   int tickets[], tempTickets[], ticketsH[], tempTicketsH[];
   ArrayResize(ticketsH, 0);
   ArrayResize(tempTickets, 0);
   ArrayResize(tempTicketsH, 0);
   
   int ticketCount = FillArrWithTickets(tempTickets, MagicNumber, Symbol(), MODE_ALL, MODE_TRADES);
   int ticketCountH = FillArrWithTickets(tempTicketsH, MagicNumberHedge, Symbol(), MODE_ALL, MODE_TRADES);
   //int compCount = FillArrWithTickets(compTrades, MagicNumberCompensatoryM1, Symbol(), curPhase, MODE_TRADES);
   
   ticketCount = IntAddIntArr(tempTickets, tempTicketsH);
      
   return(ticketCount);
}